<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;

class Area extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the City that owns the Area.
     */
    public function city()
    {
        return $this->belongsTo('App\City')->with(['region'=>function($query){$query->with('country');}]);
    }


   
    /**
     * Get the Agency that owns the Area.
     */
    public function agency()
    {
        return $this->belongsTo('App\Agency');
    }
    /**
     * Get the Agency that owns the Area.
     */
    public function region()
    {
        return $this->belongsTo('App\Region','region_id','id');
    }
    /***
     * Area Product Target Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */
    public function products_target()
    {
        return $this->hasMany('App\AreaProductTarget', 'area_id');
    }
    /***
     * Area Revisit  Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */
    public function area_revisit()
    {
        return $this->hasMany('App\AreaRevisit', 'area_id');
    }
    /***
     * Area Revisit  Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */
    public function sub_areas()
    {
        return $this->hasMany('App\SubArea', 'area_id');
    }
    /***
     * List Areas
     *
     * @return mixed
     */
    public function listAreas(Request $request)
    {
        $data = $this::orderBy('id', 'Desc');
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('name', 'like', '%' . $request->search_name . '%');
            });
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update Area
     *
     * @return mixed
     */
    public function InsertOrUpdateArea(Request $request,$id=0)
    {
        $new=true;
        if($id>0){
            $area=$this->findOrFail($id);
            $new=false;
        }else{
            $area=new $this();
        }

        $area->country_id= $request->input('country');
        $area->region_id= $request->input('region');
        $area->city_id= $request->input('city');
        $area->shop_id= $request->input('distributor');
        $area->name= $request->input('name');
        if($area->save()){
            // Area Revisit
            if(!empty($request->input('re_visit_type')) ){
                //Delete old Area Revisit
                if($new==false){
                    AreaRevisit::where('area_id',$area->id)->where('revisit_type_id', '!=',$request->input('re_visit_type'))->delete();
                }
                // add/modify Area Revisit
                if($request->input('re_visit_type')==1){
                    $revisit=AreaRevisit::where('area_id',$area->id)->where('revisit_type_id',$request->input('re_visit_type'))->first();
                    if(empty($revisit))
                    $revisit=new AreaRevisit();
                    $revisit->area_id=$area->id;
                    $revisit->revisit_type_id=$request->input('re_visit_type');
                    $revisit->value=$request->input('re_visit');
                    $revisit->save();
                }elseif ($request->input('re_visit_type')==2){
                    $old_revisit_ids=[];
                    if($new==false){
                        $revisit_ids=AreaRevisit::select(['id','value'])->where('area_id',$area->id)->where('revisit_type_id',$request->input('re_visit_type'))->get();
                        foreach ($revisit_ids as $revisit_id){
                            $old_revisit_ids[]= $revisit_id['id'];
                        }
                    }

                    if(count($request->input('week'))>0){
                        foreach ($request->input('week') as $day){
                            if(!empty($day)){
                                $revisit=AreaRevisit::where('area_id',$area->id)->where('value',$day)->where('revisit_type_id',$request->input('re_visit_type'))->first();
                                if(empty($revisit)){
                                    $revisit=new AreaRevisit();
                                }else{
                                    if (($key = array_search($revisit['id'], $old_revisit_ids)) !== FALSE) {
                                        unset($old_revisit_ids[$key]);
                                    }
                                }
                                $revisit=new AreaRevisit();
                                $revisit->area_id=$area->id;
                                $revisit->revisit_type_id=$request->input('re_visit_type');
                                $revisit->value=$day;
                                $revisit->save();
                            }
                        }
                    }
                    if(count($old_revisit_ids)>0){
                        foreach ($old_revisit_ids as $id){
                            $old_revisit=AreaRevisit::findOrFail($id);
                            $old_revisit->delete();
                        }
                    }

                }elseif ($request->input('re_visit_type')==3 && !empty($request->input('month_days'))){
                    $old_revisit_ids=[];
                    if($new==false){
                        $revisit_ids=AreaRevisit::select(['id','value'])->where('area_id',$area->id)->where('revisit_type_id',$request->input('re_visit_type'))->get();
                        foreach ($revisit_ids as $revisit_id){
                            $old_revisit_ids[]= $revisit_id['id'];
                        }
                    }
                    $month_days=explode(',',$request->input('month_days'));
                    if(count($month_days)>0){
                        foreach ($month_days as $month_day){
                            $revisit=AreaRevisit::where('area_id',$area->id)->where('value',$month_day)->where('revisit_type_id',$request->input('re_visit_type'))->first();
                            if(empty($revisit)){
                                $revisit=new AreaRevisit();
                            }else{
                                if (($key = array_search($revisit['id'], $old_revisit_ids)) !== FALSE) {
                                    unset($old_revisit_ids[$key]);
                                }
                            }
                            $revisit->area_id=$area->id;
                            $revisit->revisit_type_id=$request->input('re_visit_type');
                            $revisit->value=$month_day;
                            $revisit->save();
                        }
                    }
                    if(count($old_revisit_ids)>0){
                        foreach ($old_revisit_ids as $id){
                            $old_revisit=AreaRevisit::findOrFail($id);
                            $old_revisit->delete();
                        }
                    }


                }

            }

            // sales officer in area added
            if($new==false){
                UserArea::where('area_id',$area->id)->delete();
            }
            if(count($request->input('sales_officers'))>0){
                foreach ($request->input('sales_officers') as $sale_area){
                    $user_area = new UserArea();
                    $user_area->user_id = $sale_area;
                    $user_area->area_id = $area->id;
                    $user_area->save();
                }
            }

            //Area Product Target added

            if(count($request->input('products'))>0){
                $area_id=[];
                    $areaProductTarget=AreaProductTarget::select('id')->where('area_id',$area->id)->get()->toarray();
                    foreach ($areaProductTarget as $product){
                        if(isset($product['id']) && !empty($product['id'])){
                            $area_id[]=$product['id'];
                        }
                    }

                foreach ($request->input('products') as $value){
                    if(isset($value['id']) && !empty($value['id'])){
                        if (($key = array_search($value['id'], $area_id)) !== FALSE) {
                            unset($area_id[$key]);
                        }
                        $target_product=AreaProductTarget::findOrFail($value['id']);
                    }else{
                        $target_product=new AreaProductTarget();
                    }

                    $target_product->area_id=$area->id;
                    $target_product->product_id=$value['product'];
                    $target_product->grade_id=$value['grade'];
                    $target_product->threshold=$value['threshold'];
                    $target_product->save();
                }
                if(count($area_id)>0){
                    foreach ($area_id as $id){
                        $old_order_product=AreaProductTarget::findOrFail($id);
                        $old_order_product->delete();
                    }
                }
            }
            //Area SubArea added
            if(count($request->input('areas'))>0){
                $sub_area_id=[];
                $areaSub=SubArea::select('id')->where('area_id',$area->id)->get()->toarray();
                foreach ($areaSub as $sub){
                    if(isset($sub['id']) && !empty($sub['id'])){
                        $sub_area_id[]=$sub['id'];
                    }
                }
                foreach ($request->input('areas') as $value){
                    if(isset($value['id']) && !empty($value['id'])){
                        if (($key = array_search($value['id'], $sub_area_id)) !== FALSE) {
                            unset($sub_area_id[$key]);
                        }
                        $sub_area=SubArea::findOrFail($value['id']);
                    }else{
                        $sub_area=new SubArea();
                    }
                    $sub_area->area_id=$area->id;
                    $sub_area->name=$value['area'];
                    $sub_area->save();
                }
                if(count($sub_area_id)>0){
                    foreach ($sub_area_id as $id){
                        $old_order_product=SubArea::findOrFail($id);
                        $old_order_product->delete();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
}
