<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Database\Eloquent\SoftDeletes;
use Auth;
use Mail;
class Order extends Model
{
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the Images of the order.
     */
    public function OrderImages() {

        return $this->hasMany('App\Orderimages');
    }
    public function book_by()
    {
        return $this->belongsTo('App\User','booked_by');
    }
    /**
     * Get the Agency that owns the order.
     */
    public function agency()
    {
        return $this->belongsTo('App\Agency','agency_id');
    }
    /**
     * Get the Shop that owns the order.
     */
    public function shop()
    {
        return $this->belongsTo('App\Shop','shop_id','id');
    }
    /***
     * Order Product Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */
    public function products()
    {
        return $this->hasMany('App\OrderProduct', 'order_id')->with('OrderProductCategories','products');
    }

    /***
     * ListBooked Orders
     *
     * @return mixed
     */
    public function listBookedOrders(Request $request)
    {
        $data = $this::where('agency_id',Auth::user()->agency_id)->where('status','Booked')->orderBy('id', 'Desc')->with(['book_by']);
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('payment', '=', $request->search_name);
            });
        }
        if ($request->sale_officer != '') {
            $data->where('booked_by', '=', $request->sale_officer);
        }
        if ($request->country != '') {
            $data->where('country_id', '=', $request->country);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * List Orders
     *
     * @return mixed
     */
    public function listOrders(Request $request,$type='')
    {
        $data=null;

        if($type=='Processed' || $type==''){
            $data = $this::where('agency_id',Auth::user()->agency_id)->where('status','Processed')->orderBy('id', 'Desc')->with(['book_by']);
        }elseif ($type=='Delivered'){
            $data = $this::where('agency_id',Auth::user()->agency_id)->where('status','Delivered')->whereNotIn('type', ['Sample'])->orderBy('id', 'Desc')->with(['book_by']);
        }elseif ($type=='Cancelled'){
            $data = $this::where('agency_id',Auth::user()->agency_id)->where('status','Cancelled')->orderBy('id', 'Desc')->with(['book_by']);
        }elseif ($type=='Rejected'){
            $data = $this::where('agency_id',Auth::user()->agency_id)->where('status','Rejected')->orderBy('id', 'Desc')->with(['book_by']);
        }elseif ($type=='Returned'){
            $data = $this::where('agency_id',Auth::user()->agency_id)->where('status','Returned')->orderBy('id', 'Desc')->with(['book_by']);
        }

        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('payment', '=', $request->search_name);
            });
        }
        if ($request->sale_officer != '') {
            $data->where('booked_by', '=', $request->sale_officer);
        }
        if ($request->country != '') {
            $data->where('country_id', '=', $request->country);
        }
        if(!empty($data)){
            return $data->paginate(session()->get('site_settings')['results_per_page']);
        }else{
            return [];
        }

    }
    /***
     * Insert/Update Order
     *
     * @return mixed
     */
    public function InsertOrUpdateOrder(Request $request,$id=0)
    {
        $new=0;
        if($id>0){
            $order=$this->findOrFail($id);
            $new=1;
        }else{
            $order=new $this();
            $order->uuid= generate_uuid();
        }

        $order->agency_id =  Auth::user()->agency_id;
        $order->shop_id= $request->input('shop');
        $order->price= $request->input('price') ;
        if(!empty($request->input('discount'))){
            $order->discount= $request->input('discount');
        }
        $order->payment= $request->input('payment');
        $order->booked_by= $request->input('booked_by');
        if(!empty($request->input('booked_by'))){
            $userData=User::select(['id','country_id'])->where('id',$request->input('booked_by'))->first();
            if(!empty($userData)){
                $order->country_id=$userData->country_id;
            }
        }

        $order->date_to_deliver= $request->input('date_to_deliver');
        $order->delivery_time= $request->input('delivery_time');
        $order->type= $request->input('type');
        $order->status= $request->input('status');
        $order->latitude= $request->input('latitude');
        $order->longitude= $request->input('longitude');
        $order->remarks= $request->input('remarks');
        if($order->save()){
            // Opening balance update
            if(($request->input('status')=="Delivered" || $request->input('status')=="Returned") && !empty($request->input('shop')) && $request->input('type')!="Sample"){
              $shop=Shop::where('id',$request->input('shop'))->first();

              if($request->input('status')=="Returned"){
                  $old_balance = $shop['balance']*-1;
                  $shop->balance=($shop->balance)+($request->input('payment'));
                  $order->price= $request->input('price') * -1 ;
                  $order->payment= $request->input('payment')  * -1;
                  $shop->save();
                  $order->balance= $shop->balance;
                  $order->save();
              } else{
                  $old_balance = $shop['balance']*-1;
                  $shop->balance=($shop->balance)-($request->input('payment'));
                  $shop->save();
                  $order->balance= $shop->balance;
                  $order->save();
                  // add points on first order add
                 $f_order= Order::where('shop_id',$request->input('shop'))->get();
                  if(count($f_order)==1){
                      $so_point=new SoPoint();
                      $so_point->user_id= $request->input('booked_by');
                      $so_point->point_type_id= 2;
                      $so_point->date= $request->input('date_to_deliver');
                      $so_point->save();
                  }
                  //stock update

              }
           // Generate invoice #
               $generate= Order::max('invoice_no');
                if(!empty($generate)){
                    $order->invoice_no= $generate+1;
                    $order->save();
                }else{
                    $order->invoice_no= 1;
                    $order->save();
                }
           // Send Email on order delivered
              $new_balance =  $order->payment;
              $total_balance = ($shop->balance)*-1;
              $customer_mail = $shop['email'];
              $customer_name = $shop['name'];
             if(count($request->input('products'))>0){
                 $order_products = $request->input('products');
             }else{
                 $order_products=0;
             }


              $date = date('Y-m-d');

              if(empty($customer_mail) || url('')=='http://demo.biocospk.com') {
                  Mail::send('email.deliver_order', compact('old_balance', 'new_balance','total_balance','date','customer_name','order_products'), function($message) use ($customer_mail){
                      $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
                      $message->to('shakir130@hotmail.com')->subject('Order Delivered!');
                  });
              }else{
                  Mail::send('email.deliver_order', compact('old_balance', 'new_balance','total_balance','date','customer_name','order_products'), function($message) use ($customer_mail){
                      $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
                      $message->to($customer_mail)->subject('Order Delivered!');
                  });
              }
            }

            //Order Products added
            if(count($request->input('products'))>0){
                $order_id=[];
                $all_product=OrderProduct::select('id')->where('order_id',$order->id)->get()->toarray();
                foreach ($all_product as $product){
                    if(isset($product['id']) && !empty($product['id'])){
                        $order_id[]=$product['id'];
                    }
                }
                $is_discounted=false;
                foreach ($request->input('products') as $value){
                    if(isset($value['id']) && !empty($value['id'])){
                        if (($key = array_search($value['id'], $order_id)) !== FALSE) {
                            unset($order_id[$key]);
                        }
                        $order_product=OrderProduct::findOrFail($value['id']);
                    }else{
                        $order_product=new OrderProduct();
                    }
                    if(!empty($value['discount_price'])){
                        $is_discounted=true;
                    }
                    $order_product->order_id=$order->id;
                    $order_product->product_id=$value['product'];
                    $order_product->quantity=$value['quantity'];
                    if($value['product']==12)
                        $order_product->quantity=0;
                    if(isset($value['is_scheme']) && $value['is_scheme']==true){
                            $free=0;
                            if($value['scheme_quantity']!=null)
                            $free=(int) ($value['quantity']/$value['scheme_quantity']);
                            $free=$free*$value['scheme_free_quantity'];
                            $order_product->scheme_free_quantity=$free;
                    }
                    if(isset($value['discount_price'])){
                        $order_product->product_discounted_price=$value['discount_price'];
                    }

                    $order_product->product_actual_price=round($value['actual_price'],2);

                    $order_product->save();

                    // Order Product Category Added
                    if($value['product']==12){
                        if(count($value['type'])>0){
                            $all_quantity=0;
                            foreach($value['type'] as $product_category){
                                if(!empty($product_category['quantity'])){
                                    $all_quantity=$all_quantity+$product_category['quantity'];
                                    if(!empty($product_category['old_id']) && $product_category['old_id']!=0){
                                        $product_cat= OrderProductCategory::where('id',$product_category['old_id'])->first();
                                        if(empty($product_cat))
                                            $product_cat=new OrderProductCategory();
                                    }else{
                                        $product_cat=new OrderProductCategory();
                                    }
                                    $product_cat->order_product_id=$order_product->id;
                                    $product_cat->product_category_id=$product_category['id'];
                                    $product_cat->quantity=$product_category['quantity'];
                                    $product_cat->save();
                                    $order_product->product_actual_price=$product_category['price'];
                                    $order_product->save();
                                }
                            }
                            $order_product->quantity=$all_quantity;
                            $order_product->save();
                        }
                    }

                    // stock update
                    if(($request->input('status')=="Delivered" || $request->input('status')=="Returned") && !empty($request->input('shop')) && $request->input('type')!="Sample"){
                        $so_stock = SoStock::where('so_id',$order->booked_by)->where('product_id',$order_product->product_id)->first();
                        if($so_stock != '') {
                        if($request->input('status')=="Delivered"){
                                $old_stock = $so_stock['qty'];
                                $new_stock = $order_product->quantity;
                                $total_stock = $old_stock - $new_stock;
                                SoStock::where('so_id', $order->booked_by)->where('product_id', $order_product->product_id)
                                    ->update(array('qty' => $total_stock));
                            }elseif($request->input('status')=="Returned"){
                            $old_stock = $so_stock['qty'];
                            $new_stock = $order_product->quantity;
                            $total_stock = $old_stock + $new_stock;
                            SoStock::where('so_id', $order->booked_by)->where('product_id', $order_product->product_id)
                                ->update(array('qty' => $total_stock));
                        }
                        }
                    }

                    }
                if($is_discounted==true){
                    $order->is_discounted=true;
                    $order->save();
                }else{
                    $order->is_discounted=false;
                    $order->save();
                }
                if(count($order_id)>0){
                    foreach ($order_id as $id){
                        $old_order_product=OrderProduct::findOrFail($id);
                        $old_order_product->delete();
                    }
                }
            }

            //Order Images added
            if($new==1){
                $old_image_id=[];
                $all_product=OrderImages::select('id')->where('order_id',$order->id)->get()->toarray();
                if(count($all_product)>0){
                    foreach ($all_product as $all){
                        if(isset($all['id']) && !empty($all['id'])){
                            $old_image_id[]=$all['id'];
                        }
                    }
                }
                if(count($request->input('old_images_id'))>0){
                    foreach ($request->input('old_images_id') as $value){
                        if (($key = array_search($value, $old_image_id)) !== FALSE) {
                            unset($old_image_id[$key]);
                        }
                    }
                }
                if(count($old_image_id)>0){
                    foreach ($old_image_id as $id){
                        $old_order_product=OrderImages::findOrFail($id);
                        $old_order_product->delete();

                        $file = public_path() .str_replace(url(''),"",$old_order_product['image']);
                        if (file_exists($file)) {
                            @unlink($file);
                        }
                    }
                }
            }
            for($i=-1,$j=0;$i<12;$i++,$j++){
                if ($request->hasFile('image'.$j)) {
                    $filename = 'image-' . time() . rand(99, 199) . '.' . $request->file('image'.$j)->getClientOriginalExtension();
                    $request->file('image'.$j)->move(config('constants.UPLOAD_PATH') . 'orders/', $filename);
                    if (isset($filename)){
                        $pro_image=new OrderImages();
                        $pro_image->image =  url('/uploads/orders/'.$filename);
                        $pro_image->order_id = $order->id;
                        $pro_image->save();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
}
