<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;

class Target extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the user that owns the target.
     */
    public function rsm()
    {
        return $this->belongsTo('App\User','rsm_id','id');
    }
    /***
     * Order Product Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */
    public function target_details()
    {
        return $this->hasMany('App\TargetDetail', 'target_id')->with('user');
    }
    // CMonth Recovery
    public function CMonthRecovery($user_id) {
        $payments=Payment::where('user_id',$user_id)
            ->where('payment_type', 'Cash')
            ->whereBetween('promise_cheque_date',[date('Y-m-01'),date("Y-m-d")])
            ->sum('amount');
        return $payments;
    }
    //CMonth Sales
    public function CMonthSales($user_id) {
        $orders=Order::where('booked_by',$user_id)
            ->where('status','Delivered')
            ->where('type', '!=', 'Sample')
            ->whereBetween('date_to_deliver',[date('Y-m-01'),date("Y-m-d")])
            ->sum('payment');
        return $orders;
    }

    /***
     * List Target
     *
     * @return mixed
     */
    public function listTargets(Request $request)
    {
        $data = $this::orderBy('id', 'Desc');
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('date', 'like', '%' . $request->search_name . '%');
            });
        }
        if ($request->sale_officer != '') {
            $data->where('rsm_id', '=', $request->sale_officer);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update
     *
     * @return mixed
     */
    public function InsertOrUpdate(Request $request,$id=0)
    {
        $new=true;
        if($id>0){
            $target=$this->findOrFail($id);
            $new=false;
        }else{
            $target=new $this();
        }
        $target->rsm_id= $request->input('rsm');
        $target->date_start= $request->input('date_start');
        $target->date_end= $request->input('date_end');
        $target->sales_target= $request->input('sales_target');
        $target->recovery_target= $request->input('recovery_target');
        if($target->save()){
            //Asm,Ase & So target added
            TargetDetail::where('target_id',$target->id)->delete();
            $start=$request->input('date_start');
            $end=$request->input('date_end');
            // count b/w dates
            $st=substr($request->input('date_start'), strpos($request->input('date_start'), "-") + 1);
            $ed=substr($request->input('date_end'), strpos($request->input('date_end'), "-") + 1);
            $count=1;
            if(abs($st - $ed)>0){
                $count=abs($st - $ed)+1;
            }
            for ($start;$start<=$end;$start++){
                if(count($request->input('d_targets'))>0){
                    foreach ($request->input('d_targets') as $value){
                        $d_target=new TargetDetail();
                        $d_target->target_id=$target->id;
                        $d_target->user_id=$value['user_id'];
                        $d_target->date=$start;
                        $d_target->sales_target=($value['sales_target'])/$count;
                        $d_target->recovery_target=($value['recovery_target'])/$count;
                        $d_target->save();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
}
