<?php

namespace App;

use Carbon\Carbon;
use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Zizaco\Entrust\Traits\EntrustUserTrait;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Auth;

//class User extends Authenticatable
class User extends Authenticatable
{
    use Notifiable;
    use EntrustUserTrait;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name', 'email', 'password',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];
    // Append attribute
    protected $appends = [
        'role_user',
    ];
    /**
     * Get the Shop that owns the order.
     */
    public function supplier()
    {
        return $this->hasOne('App\SupplierSo','so_id');
    }
    /**
     * Get the Feedback of the user.
     */
    public function feedbacks() {
        return $this->hasMany('App\Feedback','user_id','id');
    }
    /**
     * Get the Region of the user.
     */
    public function region() {
        return $this->belongsTo('App\Region','region_id','id');
    }
    /**
     * Get the country of the user.
     */
    public function country() {
        return $this->belongsTo('App\Country','country_id','id');
    }
    /**
     * Get the shops of the user.
     */
    public function shops() {
        return $this->hasMany('App\Shop','sale_officer_id','id');
    }
    /**
     * Get the avalable Stock of the user.
     */
    public function avalableStocks() {
        return $this->hasMany('App\SoStock','so_id','id')->with('product');
    }
    /**--
     * Get the soLedger of the user.
     */
    public function soLedger() {
        return $this->hasMany('App\UserExpenseDetail','user_id','id');
    }
    /**
     * Get the Orders of the user.
     */
    public function orders() {
        return $this->hasMany('App\Order','booked_by','id');
    }
    public function sols() {
        return $this->hasOne('App\Voucher','Distributor_id','id');
    }

      public function dfse() {
        return $this->hasOne('App\Dfs','booked_by','id');
    }
       

    

    // role user append
    public function getRoleUserAttribute()
    {
       $role= UserRole::select('user_id','role_id')->where('user_id',$this->id)->first();
       return 123;
        if(!empty($role->role_id)){
            $role_name=Role::select('id','name','display_name')->where('id',$role->role_id)->first();
            if(!empty($role_name) && !empty($role_name->display_name)){
                return $role_name->display_name;
            }else{
                return "";
            }
        }else{
            return "";
        }

    }
    // CMonth Sales And Recovery
    public function CMonthSalesAndRecovery($user_id) {
        $tenures=[];
        $size=0;
        $payments=Payment::where('user_id',$user_id)
            ->where('payment_type', 'Cash')
            ->whereBetween('promise_cheque_date',[date('Y-m-01'),date("Y-m-d")])
            ->get();
        $orders=Order::where('booked_by',$user_id)
            ->where('status','Delivered')
            ->where('type', '!=', 'Sample')
            ->whereBetween('date_to_deliver',[date('Y-m-01'),date("Y-m-d")])
            ->get();
        if(isset($orders) && count($orders)>0){
            foreach ($orders as $key=>$order){
                $tenures[$key]=$order;
                $tenures[$key]['date']=$order['date_to_deliver'];
                $tenures[$key]['ledger_type']='order';

            }
            $size=count($orders);
        }
        if(isset($payments) && count($payments)>0){
            foreach ($payments as $index=>$payment){
                $tenures[$size]=$payment;
                $tenures[$size]['date']=$payment['promise_cheque_date'];
                $tenures[$size]['ledger_type']='payment';
                $size++;
            }
        }
        usort($tenures, function($a, $b) {
            return strtotime($a['date']) - strtotime($b['date']);
        });
        return $tenures;
    }
    /**
     * Get the City of the user.
     */
    public function city() {
        return $this->belongsTo('App\City','city_id','id');
    }
    // get current month sales
    public function CMonthSale($user_id)
    {
        return Order::where('booked_by',$user_id)
            ->where('status','Delivered')
            ->whereBetween('date_to_deliver',[date('Y-m-01'),date("Y-m-d")])
            ->where('type', '!=', 'Sample')
            ->sum('payment');
    }
    // get current month Recovery
    public function CMonthRecovery($user_id)
    {
        return Payment::where('user_id',$user_id)
            ->whereBetween('promise_cheque_date',[date('Y-m-01'),date("Y-m-d")])
            ->where('payment_type', 'Cash')
            ->sum('amount');
    }
    // get CMonthExpense
    public function CMonthExpense($user_id)
    {
        return UserExpense::where('user_id',$user_id)
            ->whereBetween('date',[date('Y-m-01'),date("Y-m-d")])
            ->get();
    }
    /***
     * List Users
     *
     * @return mixed
     */
    public function listUsers(Request $request)
    {
        $data="";
        if ($request->role != '') {
            $users_id=UserRole::select(['user_id'])->where('role_id',$request->role)->get();
            if(count($users_id)>0){
                $ids=[];
                foreach ($users_id as $value){
                    if(!empty($value['user_id'])){
                        $ids[]=$value['user_id'];
                    }
                }
                $data = $this::where('agency_id',Auth::user()->agency_id)->whereIn('id',$ids)->where('id','!=',95)->orderBy('id', 'Desc');
            }
        }else{
            $data = $this::where('agency_id',Auth::user()->agency_id)->where('id','!=',95)->orderBy('id', 'Desc');
        }
        if ($request->search != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('id', '=',$request->search);
            });
        }
        if ($request->country != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('country_id', '=',$request->country);
            });
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * InsertOrUpdate User
     *
     * @return mixed
     */
    public function InsertOrUpdateUser(Request $request,$id=0)
    {
        $new=true;
        if($id>0){
            $user=$this->findOrFail($id);
            $new=false;
        }else{
            $user=new $this();
            $user->email = $request->input('email');
        }
        if(!empty($request->input('parent_id'))){
            $user->parent_id =  $request->input('parent_id');
        }else{
            $user->parent_id =  Auth::user()->id;
        }
        if(!empty($request->input('password'))){
            $user->password = bcrypt($request->input('password'));
        }
        if($request->input('parent_type')=="both" ){
            $user->parent_id =  $request->input('rsm');
            $user->parent_id_2 =  $request->input('asm');
        }else{
            $user->parent_id_2 =  null;
        }
        $user->firstname = $request->input('firstname');
        $user->lastname = $request->input('lastname');
        $user->username = $request->input('username');
        $user->father_name = $request->input('father_name');
        $user->country_id = $request->input('country');
        $user->region_id= $request->input('region');
        $user->city_id= $request->input('city');
        $user->address = $request->input('address');
        $user->religion = $request->input('religion');
        if(!empty($request->input('parent_type'))){
            $user->parent_type = $request->input('parent_type');
        }
        $user->gender = $request->input('gender');
        if($request->input('is_zsm')){
            $user->is_zsm = true;
        }else{
            $user->is_zsm = false;
        }
        // rsm_id fetch
        switch ($request->input('role_id')){
            case 3:
                $user->rsm_id=$request->input('parent_id');
                break;
            case 4:
                if($request->input('parent_type')=='rsm'  ){
                    $user->rsm_id=$request->input('parent_id');
                }elseif ( $request->input('parent_type')=='asm'){
                    $asm=User::select('id','parent_id','rsm_id')->where('id',$request->input('parent_id'))->first();
                    if(!empty($asm->parent_id))
                    $user->rsm_id=$asm->parent_id;
                }elseif ( $request->input('parent_type')=='ase'){
                    $asm=User::select('id','parent_id','rsm_id')->where('id',$request->input('parent_id'))->first();
                    if(!empty($asm->rsm_id)){
                        $user->rsm_id=$asm->rsm_id;
                    }elseif(!empty($asm->parent_id)){
                        $asm2=User::select('id','parent_id','rsm_id')->where('id',$asm->parent_id)->first();
                        if(!empty($asm2->rsm_id)){
                            $user->rsm_id=$asm2->rsm_id;
                        }elseif(!empty($asm2->parent_id)){
                            $user->rsm_id=$asm2->parent_id;
                        }
                    }
                }elseif ($request->input('parent_type')=='both'){
                    $user->rsm_id=$request->input('rsm');
                }
                break;
            case 9:
                if($request->input('parent_type')=='rsm'){
                    $user->rsm_id=$request->input('parent_id');
                }elseif($request->input('parent_type')=='asm'){
                    $asm=User::where('id',$request->input('parent_id'))->first();
                    if(!empty($asm['parent_id']))
                        $user->rsm_id=$asm['parent_id'];
                }elseif ($request->input('parent_type')=='both'){
                    $user->rsm_id=$request->input('rsm');
                }
                break;
            default:
                $user->rsm_id =null;
        }
        $user->agency_id =  Auth::user()->agency_id;
        $user->cnic = $request->input('cnic');
        $user->phone_1 = $request->input('phone_1');
        $user->phone_2 = $request->input('phone_2');
        $user->landline = $request->input('landline');
        $user->dob = date('Y-m-d', strtotime($request->input('dob')));
        $user->joining_date = date('Y-m-d', strtotime($request->input('joining_date')));
        $user->status = $request->status=='0' ? 0 : 1;
        if ($request->hasFile('image')) {
           
            $filename = 'image-' . time() . rand(99, 199) . '.' . $request->file('image')->getClientOriginalExtension();
           
            
            $request->file('image')->move(config('constants.UPLOAD_PATH') . 'users/', $filename);
            if($new==false){
                
                $search = config('constants.IMAGE_DIR') .'users/';
               
                $path    = str_replace("$search", " ",$user->image);
                 
                  $file = public_path().str_replace(url(''),"",$user->image);
                
               
                if (file_exists($file)) {
                    @unlink($file);
                }
                
            }
        }
        if (isset($filename))
            $user->image = url('/uploads/users/'.$filename);
            
        if($user->save()){
            if($user->id!=9){
                $role = Role::findOrFail($request->role_id);
                $user->detachRoles();
                $user->attachRole($role);
            }
            if(($request->role_id==4 || $request->role_id==9) && !empty($request->input('supplier'))){
                $su=SupplierSo::where('so_id',$user->id)->first();
                if(empty($su))
                    $su=new SupplierSo();
                $su->so_id=$user->id;
                $su->supplier_id=$request->input('supplier');
                $su->save();
            }
            if($new==true){
                $obj = new UserToken();
                $obj->token = generate_uuid ();
                $obj->user_id = $user->id;
                $obj->status = 'active';
                $obj->issue_date = Carbon::now ();
                $obj->save ();
            }else{
                if(!empty($request->input('password'))){

                   UserToken::where('user_id',$user->id)->update(array('token'=>generate_uuid ()));
                }
            }
            return true;
        }else{
            return false;
        }
    }
    /***
     * Create or update user
     *
     * @param $data
     * @return mixed
     */
    public function createOrUpdateUser($data)
    {
        $new=false;
        if(array_key_exists('email', $data)) {
            $user = $this->where('email', '=', $data['email'])
                ->first();
        }
        if(is_null($user)) {
            $new=true;
            $user = new User();
            if(array_key_exists('email', $data)) {
                $user->email = $data['email'];
            }
        }
        $splitName = split_name($data['name']);
        if(count($splitName) > 1) {
            $user->firstname = $splitName[0];
            $user->lastname  = $splitName[1];
        }
        $user->address  =  $data["address"];
        $user->landline =  $data['landline'];
        $user->phone_1  =  $data['phone_1'];
        $user->phone_2  =  $data['phone_2'];
        $user->save();
        if($new && $new==true){
            $role = \App\Role::findOrFail(7);
            $user = \App\User::findOrFail($user->id);
            $user->attachRole($role);
        }
        return $user->id;
    }
    public function IsRole($query, $role) {
        return $query->whereHas(
            'roles', function($query) use ($role){
            $query->where('name', $role);
        }
        );
    }
}
