<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;

class UserExpense extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the Agency that owns the Area.
     */
    public function agency()
    {
        return $this->belongsTo('App\Agency');
    }
    public function user()
    {
        return $this->belongsTo('App\User','user_id');
    }
    public function expense()
    {
        return $this->belongsTo('App\Expense','expense_id');
    }
    /**
     * Get the Images of the userExpense.
     */
    public function Images() {
        return $this->hasMany('App\ExpenseImage','user_expense_id','id');
    }
    /***
     * List Expense
     *
     * @return mixed
     */
    public function listExpense(Request $request)
    {
        $data = $this::orderBy('id', 'Desc');
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('amount', 'like', '%' . $request->search_name . '%');
            });
        }
        if ($request->user != '') {
            $data->where('user_id', '=', $request->user);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update
     *
     * @return mixed
     */
    public function InsertOrUpdate(Request $request,$id=0)
    {
        $new=0;
        if($id>0){
            $expense=$this->findOrFail($id);
            $new=1;
        }else{
            $expense=new $this();
        }
        $expense->user_id= $request->input('user');
        $expense->expense_id= $request->input('expense_type');
        $expense->date= $request->input('date');
        $expense->amount= $request->input('amount');
        if($expense->save()){
            if($new==1){
                // User Expense Images
                $old_image_id=[];
                $all_product=ExpenseImage::select('id')->where('user_expense_id',$expense->id)->get()->toarray();
                if(count($all_product)>0){
                    foreach ($all_product as $all){
                        if(isset($all['id']) && !empty($all['id'])){
                            $old_image_id[]=$all['id'];
                        }
                    }
                }
                if(count($request->input('old_images_id'))>0){
                    foreach ($request->input('old_images_id') as $value){
                        if (($key = array_search($value, $old_image_id)) !== FALSE) {
                            unset($old_image_id[$key]);
                        }
                    }
                }
                if(count($old_image_id)>0){
                    foreach ($old_image_id as $id){
                        $old_order_product=ExpenseImage::findOrFail($id);
                        $old_order_product->delete();
                        $file = public_path() .str_replace(url(''),"",$old_order_product['image']) ;
                        if (file_exists($file)) {
                            @unlink($file);
                        }
                    }
                }
                // SO Balance Adjustment
                $diff = $request->input('old_amount') - $request->input('amount');
                $so_balance=User::where('user_id',$expense->user_id)->first();
                $so_balance->balance=($so_balance->balance) - ($diff);
                $so_balance->save();
                // So ledger handle
                $soLedger= UserExpenseDetail::where('date',$expense->date)
                    ->where('user_id',$expense->user_id)
                    ->first();
                if(empty($soLedger))
                $soLedger=new UserExpenseDetail();
                $soLedger->user_id=$expense->user_id;
                $soLedger->type="Expense";
                $soLedger->amount=$expense->amount;
                $soLedger->balance=$so_balance->balance;
                $soLedger->date=$expense->date;
                $soLedger->save();
            }else{
                // So Balance Adjustment
                $so_balance=User::where('user_id',$expense->user_id)->first();
                $so_balance->balance=($so_balance->balance) - ($expense->amount);
                $so_balance->save();
                // So ledger handle
                $soLedger=new UserExpenseDetail();
                $soLedger->user_id=$expense->user_id;
                $soLedger->type="Expense";
                $soLedger->amount=$expense->amount;
                $soLedger->balance=$so_balance->balance;
                $soLedger->date=$expense->date;
                $soLedger->save();
            }
            for($i=-1,$j=0;$i<12;$i++,$j++){
                if ($request->hasFile('image'.$j)) {
                    $filename = 'image-' . time() . rand(99, 199) . '.' . $request->file('image'.$j)->getClientOriginalExtension();
                    $request->file('image'.$j)->move(config('constants.UPLOAD_PATH') . 'expenses/', $filename);
                    if (isset($filename)){
                        $pro_image=new ExpenseImage();
                        $pro_image->image = url('/uploads/expenses/'.$filename);
                        $pro_image->user_expense_id = $expense->id;
                        $pro_image->save();
                    }
                }
            }

            return true;
        }else{
            return false;
        }
    }
}
